/*
Log file
*/

import QtQuick 1.0

Item {
    id: settingMenu
    width: parent.width
    height: parent.height - header.height

    property alias username: usrNameInput.text
    property alias password: usrPwdInput.text
    property alias timer: updateFreqInput.text

    property bool useWidgetView: false
    property bool usePortfolioView: false
    property bool useDarkTheme: false // for AMOLED
    property bool useWebview: false // for news webview

    property int listFontSize: 28
    property int fontSize1: listFontSize - 2

    property color textInputBg: "#E2EFFC" //"#C2DEFA"
    property color breakBg: "#D5D5D5"

    // signal interface
    signal callPfo
    signal close
    signal signout
    signal signin
    signal about


    Rectangle {
        id: background
        x: 2; y: 2; width: parent.width - x*2; height: parent.height - y*2
        color: "ivory"
        border.color: "orange"
        radius: 5
    }

    About {
        id: aboutView
        anchors.fill: parent
        opacity: 0
    }


    Column{
        id: menuGrid
        x:10
        y:10
        width: parent.width - 2*x
        height:  parent.height - 2*y
        spacing: 10

        // Username + password
        Grid {
            spacing: 15
            columns: 2
            width: parent.width

            // Username
            Text {
                font.bold: true
                font.pixelSize: listFontSize
                text: "Username"
            }

            Rectangle {
                id: usrInputBG
                x: 2
                width: settingMenu.width - 20 - x
                height: usrNameInput.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                TextInput {
                    id: usrNameInput
                    width: usrInputBG.width
                    maximumLength: 30
                    focus: parent.focus
                    font.pixelSize: fontSize1
                }
            }

            // Password
            Text {
                font.bold: true
                font.pixelSize: listFontSize
                text: "Password"
            }

            Rectangle {
                id: usrPwdInputBG
                x: 2
                width: settingMenu.width - 20 - x
                height: usrPwdInput.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                TextInput {
                    id: usrPwdInput
                    width: usrPwdInputBG.width
                    maximumLength: 30
                    focus: parent.focus
                    echoMode: TextInput.Password
                    font.pixelSize: fontSize1
                }
            }
        }

        // login/logout
        Row {
            x: 20
            width: settingMenu.width
            height: pfo.height
            spacing: 50

            SimpleButton {
                id: signoutButton
                width: 40
                height: 40
                radius: 40
                textSize: 20
                textColor: "#696d71"
                text: "logout"
                onClicked: { settingMenu.signout(); }
            }

            SimpleButton {
                id: signinButton
                width: 40
                height: 40
                radius: 40
                textSize: 20

                text: "login"
                onClicked: { settingMenu.signin(); }
            }
        }

        Rectangle {
            width: settingMenu.width
            height: 2
            color: "#D5D5D5"
        }

        // Update frequency
        Row {
            spacing: 20

            // Update frequency
            Text {
                font.bold: true
                font.pixelSize: listFontSize
                text: "Refresh (Seconds)"
            }
            Rectangle {
                id: updateFreqInputBG
                x: 2
                width: settingMenu.width - 20 - x
                height: usrNameInput.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                TextInput {
                    id: updateFreqInput
                    //maximumLength: 5
                    inputMask: "99999"
                    text: "30"
                    focus: parent.focus
                    font.pixelSize: fontSize1
                }
            }
        }

        Rectangle {
            width: settingMenu.width
            height: 2
            color: "#D5D5D5"
        }

        // Portfolio view + in-app news
        Grid {
            spacing: 10
            columns: 4
            width: parent.width

            // Portfolio view
            Text {
                id: pfoView
                font.bold: true
                font.pixelSize: listFontSize
                text: "Portfolio View"
            }

            Rectangle {
                id: pfoBG
                x: 2
                width: pfoView.height
                height: pfoView.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                Rectangle {
                    id: pfoSelectBG
                    x: 4; y: 4
                    width: pfoView.height - 2*x
                    height: pfoView.height - 2*y
                    radius: 5
                    smooth: true
                    visible: settingMenu.usePortfolioView
                    color: "#7DBFFF"
                }

                MouseArea {
                    id: pfoViewMouseArea
                    anchors.fill: parent

                    onClicked: {
                        settingMenu.usePortfolioView = !settingMenu.usePortfolioView;
                        if (settingMenu.usePortfolioView)
                            settingMenu.useWidgetView = !settingMenu.usePortfolioView;
                    }
                }
            }

            // In-app news
            Text {

                font.bold: true
                font.pixelSize: listFontSize
                text: "In-app News"
            }

            Rectangle {
                id: newsBG
                x: 2
                width: pfoView.height
                height: pfoView.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                Rectangle {
                    id: newsSelectBG
                    x: 4; y: 4
                    width: pfoView.height - 2*x
                    height: pfoView.height - 2*y
                    radius: 5
                    smooth: true
                    visible: settingMenu.useWebview
                    color: "#7DBFFF"
                }

                MouseArea {
                    id: newsMouseArea
                    anchors.fill: parent

                    onClicked: { settingMenu.useWebview = !settingMenu.useWebview; }
                }
            }

            // Widget View
            Text {

                font.bold: true
                font.pixelSize: listFontSize
                text: "Widget View"
            }

            Rectangle {
                id: widgetBG
                x: 2
                width: pfoView.height
                height: pfoView.height

                color: textInputBg
                radius: 5
                border.width: 1
                border.color: "#aaa"
                smooth: true

                Rectangle {
                    id: widgetViewBG
                    x: 4; y: 4
                    width: pfoView.height - 2*x
                    height: pfoView.height - 2*y
                    radius: 5
                    smooth: true
                    visible: settingMenu.useWidgetView
                    color: "#7DBFFF"
                }

                MouseArea {
                    id: widgetViewMouseArea
                    anchors.fill: parent

                    onClicked: {
                        settingMenu.useWidgetView = !settingMenu.useWidgetView;
                        if (settingMenu.useWidgetView)
                            settingMenu.usePortfolioView = !settingMenu.useWidgetView;
                    }
                }
            }
        }

        Rectangle {
            width: settingMenu.width
            height: 2
            color: "#D5D5D5"
        }

        // Select profolio
        Row {
            x: 2
            width: settingMenu.width
            height: pfo.height
            spacing: 20

            Text {
                id: pfo
                font.bold: true
                font.pixelSize: listFontSize
                text: "Select Portfolio"
            }
            TextButton {
                id: pfoButton
                text: "Portfolio"

                onClicked: { settingMenu.callPfo(); }
            }
        }

        Rectangle {
            width: settingMenu.width
            height: 2
            color: "#D5D5D5"
        }
    }

    BackButton {
        id: backButton
        anchors { bottom: parent.bottom; left: parent.left; leftMargin: 20; bottomMargin: 20; }
        width: 65
        height: 65
        onClicked: {
            if (settingMenu.state=="shownAbout") {
                settingMenu.state = "shown";
            }
            else { settingMenu.close(); }
        }
    }

    TextButton {
        id: aboutButton
        anchors { bottom: parent.bottom; right: parent.right; rightMargin: 20; bottomMargin: 20; }
        width: 60
        height: aboutButton.width
        radius: aboutButton.width

        color: "#F2F2F2"
        lightColor: "#FFFFFF"
        pressColor: aboutButton.color

        text: "?"
        textColor: "white"
        textSize: 55
        textBold: true

        onClicked: { settingMenu.state = "shownAbout"; }
    }

    states: [
        State {
            name: "hidden"
            PropertyChanges { target: settingMenu; y: 0 - settingMenu.height }
        },
        State {
            name: "shown"
            PropertyChanges { target: settingMenu; y: header.height }
        },
        State {
            name: "shownAbout"
            PropertyChanges { target: settingMenu; y: header.height }
            PropertyChanges { target: menuGrid; opacity: 0 }
            PropertyChanges { target: aboutView; opacity: 1 }
        }
    ]

    transitions: Transition {
        SequentialAnimation {
            PropertyAnimation {
                target: settingMenu
                properties: "y"
                duration: 300
                easing.type: "OutQuad"
            }
        }
    }
}
